@@ -1,29 +1,14 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -34,11 +19,11 @@
 #include <ofono/modem.h>
 #include <ofono/call-barring.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 
-#include "atmodem.h"
-
 static const char *clck_prefix[] = { "+CLCK:", NULL };
 static const char *none_prefix[] = { NULL };
 
@@ -89,7 +74,10 @@ static void at_call_barring_query(struct ofono_call_barring *cb,
 	if (strlen(lock) != 2)
 		goto error;
 
-	snprintf(buf, sizeof(buf), "AT+CLCK=\"%s\",2", lock);
+	if (cls == 7)
+		snprintf(buf, sizeof(buf), "AT+CLCK=\"%s\",2", lock);
+	else
+		snprintf(buf, sizeof(buf), "AT+CLCK=\"%s\",2,,%d", lock, cls);
 
 	if (g_at_chat_send(chat, buf, clck_prefix,
 				clck_query_cb, cbd, g_free) > 0)
@@ -204,12 +192,12 @@ static void at_call_barring_remove(struct ofono_call_barring *cb)
 {
 	GAtChat *chat = ofono_call_barring_get_data(cb);
 
+	g_idle_remove_by_data(cb);
 	g_at_chat_unref(chat);
 	ofono_call_barring_set_data(cb, NULL);
 }
 
-static struct ofono_call_barring_driver driver = {
-	.name		= "atmodem",
+static const struct ofono_call_barring_driver driver = {
 	.probe		= at_call_barring_probe,
 	.remove		= at_call_barring_remove,
 	.set		= at_call_barring_set,
@@ -217,12 +205,4 @@ static struct ofono_call_barring_driver driver = {
 	.set_passwd	= at_call_barring_set_passwd,
 };
 
-void at_call_barring_init(void)
-{
-	ofono_call_barring_driver_register(&driver);
-}
-
-void at_call_barring_exit(void)
-{
-	ofono_call_barring_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(call_barring, atmodem, &driver)
