@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,26 +15,27 @@
 #include <errno.h>
 
 #include <glib.h>
+#include <ell/ell.h>
 
 #include <ofono/log.h>
 #include <ofono/modem.h>
 #include <ofono/voicecall.h>
 #include "vendor.h"
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 
 #include "common.h"
 
-#include "atmodem.h"
-
 /* Amount of ms we wait between CLCC calls */
 #define POLL_CLCC_INTERVAL 500
 
- /* Amount of time we give for CLIP to arrive before we commence CLCC poll */
+/* Amount of time we give for CLIP to arrive before we commence CLCC poll */
 #define CLIP_INTERVAL 200
 
- /* When +VTD returns 0, an unspecified manufacturer-specific delay is used */
+/* When +VTD returns 0, an unspecified manufacturer-specific delay is used */
 #define TONE_DURATION 1000
 
 static const char *clcc_prefix[] = { "+CLCC:", NULL };
@@ -161,6 +148,11 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 			goto poll_again;
 		}
 
+		if (vd->vendor == OFONO_VENDOR_DROID) {
+			poll_again = TRUE;
+			goto poll_again;
+		}
+
 		ofono_error("We are polling CLCC and received an error");
 		ofono_error("All bets are off for call management");
 		return;
@@ -221,9 +213,7 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 			 * CNAP doesn't arrive as part of CLCC, always
 			 * re-use from the old call
 			 */
-			strncpy(nc->name, oc->name,
-					OFONO_MAX_CALLER_NAME_LENGTH);
-			nc->name[OFONO_MAX_CALLER_NAME_LENGTH] = '\0';
+			l_strlcpy(nc->name, oc->name, sizeof(nc->name));
 			nc->cnap_validity = oc->cnap_validity;
 
 			/*
@@ -1113,7 +1103,10 @@ static int at_voicecall_probe(struct ofono_voicecall *vc, unsigned int vendor,
 	g_at_chat_send(vd->chat, "AT+CRC=1", NULL, NULL, NULL, NULL);
 	g_at_chat_send(vd->chat, "AT+CLIP=1", NULL, NULL, NULL, NULL);
 	g_at_chat_send(vd->chat, "AT+CDIP=1", NULL, NULL, NULL, NULL);
-	g_at_chat_send(vd->chat, "AT+CNAP=1", NULL, NULL, NULL, NULL);
+
+	if (vd->vendor != OFONO_VENDOR_QUECTEL &&
+			vd->vendor != OFONO_VENDOR_QUECTEL_EC2X)
+		g_at_chat_send(vd->chat, "AT+CNAP=1", NULL, NULL, NULL, NULL);
 
 	switch (vd->vendor) {
 	case OFONO_VENDOR_QUALCOMM_MSM:
@@ -1153,7 +1146,6 @@ static void at_voicecall_remove(struct ofono_voicecall *vc)
 }
 
 static const struct ofono_voicecall_driver driver = {
-	.name			= "atmodem",
 	.probe			= at_voicecall_probe,
 	.remove			= at_voicecall_remove,
 	.dial			= at_dial,
@@ -1172,12 +1164,4 @@ static const struct ofono_voicecall_driver driver = {
 	.send_tones		= at_send_dtmf
 };
 
-void at_voicecall_init(void)
-{
-	ofono_voicecall_driver_register(&driver);
-}
-
-void at_voicecall_exit(void)
-{
-	ofono_voicecall_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(voicecall, atmodem, &driver)
