@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
+ *  Copyright (C) 2009-2010  Nokia Corporation and/or its subsidiary(-ies).
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -19,9 +19,7 @@
  *
  */
 
-#include "modem.h"
-
-GIOChannel *phonet_new(GIsiModem *, uint8_t resource);
-size_t phonet_peek_length(GIOChannel *io);
-ssize_t phonet_read(GIOChannel *io, void *restrict buf, size_t len,
-			uint16_t *restrict obj, uint8_t *restrict res);
+GIOChannel *g_isi_phonet_new(unsigned int ifindex);
+size_t g_isi_phonet_peek_length(GIOChannel *io);
+ssize_t g_isi_phonet_read(GIOChannel *io, void *restrict buf, size_t len,
+				struct sockaddr_pn *addr);
